@@ -13,6 +13,7 @@
 "*fadeTicks:		20",
 "*splash:		True",
 "*splashDuration:	5",
+"*visualID:		default",
 "*captureStderr: 	True",
 "*overlayTextForeground:	#FFFF00",
 "*overlayTextBackground:	#000000",
@@ -28,6 +29,9 @@
 "*loadURL: netscape -remote 'openURL(%s)' || netscape '%s'",
 "*installColormap:	True",
 "*programs:	qix -root -solid -delay 0 -segments 100			\\n\
+		qix -root -count 4 -solid -transparent			\\n\
+		qix -root -count 5 -solid -transparent -linear		  \
+			-segments 250 -size 100				\\n\
 		attraction -root -mode balls				\\n\
 		attraction -root -mode lines -points 3 -segments 200	\\n\
 		attraction -root -mode splines -segments 300		\\n\
@@ -118,11 +122,19 @@
 		critical -root						\\n\
 		phosphor -root						\\n\
 		xmatrix -root						\\n\
+		petri -root -size 1 -count 20				\\n\
+		petri -root -minlifespeed 0.02 -maxlifespeed 0.03	  \
+		   -minlifespan 1 -maxlifespan 1 -instantdeathchan 0	  \
+		   -minorchan 0 -anychan 0.3				\\n\
+		shadebobs -root						\\n\
+    default-n:  webcollage -root					\\n\
+ -  default-n:	webcollage -root -filter 'vidwhacker -stdin -stdout'    \\n\
+ -  default-n:	vidwhacker -root					\\n\
 									  \
-	mono:	rocks -root						\\n\
+	 mono:	rocks -root						\\n\
 	color:	rocks -root -fg darksalmon				\\n\
 									  \
-	mono:	qix -root -linear -count 5 -size 200 -spread 30		  \
+	 mono:	qix -root -linear -count 5 -size 200 -spread 30		  \
 			-segments 75 -solid -xor			\\n\
 									  \
 	color:	attraction -root -mode polygons				\\n\
@@ -130,24 +142,22 @@
 	color:	attraction -root -glow -points 10			\\n\
 	color:	bubbles -root						\\n\
 									  \
-  PseudoColor:	qix -root -count 4 -solid -transparent			\\n\
-  PseudoColor:	qix -root -count 5 -solid -transparent -linear		  \
-			-segments 250 -size 100				\\n\
-									  \
-		gears -root					  	\\n\
-		superquadrics -root				  	\\n\
-		morph3d -root					  	\\n\
-		cage -root					  	\\n\
-		moebius -root					  	\\n\
-		stairs -root					  	\\n\
-		pipes -root					  	\\n\
-		sproingies -root				  	\\n\
-		rubik -root					  	\\n\
-		atlantis -root					  	\\n\
-		lament -root					  	\\n\
-		bubble3d -root					  	\\n\
-		glplanet -root					  	\\n\
-		pulsar -root					  	\\n",
+	   GL:	gears -root					  	\\n\
+	   GL:	superquadrics -root				  	\\n\
+	   GL:	morph3d -root					  	\\n\
+	   GL:	cage -root					  	\\n\
+	   GL:	moebius -root					  	\\n\
+	   GL:	stairs -root					  	\\n\
+	   GL:	pipes -root					  	\\n\
+	   GL:	sproingies -root				  	\\n\
+	   GL:	rubik -root					  	\\n\
+	   GL:	atlantis -root					  	\\n\
+	   GL:	lament -root					  	\\n\
+	   GL:	bubble3d -root					  	\\n\
+	   GL:	glplanet -root					  	\\n\
+	   GL:	pulsar -root					  	\\n\
+  -	   GL:  pulsar -root -texture -mipmap -texture_quality  	  \
+		       -light -fog					\\n",
 " ",
 "XScreenSaver.pointerPollTime:		5",
 "XScreenSaver.initialDelay:		0",
@@ -269,6 +279,7 @@
 "*Box.borderWidth:				0",
 "*Label.borderWidth:				0",
 "*preferences_dialog*Dialog.borderWidth:		0",
+"*demo_dialog*run.label:				Run",
 "*demo_dialog*next.label:			Run Next",
 "*demo_dialog*prev.label:			Run Previous",
 "*demo_dialog*edit.label:			Preferences",
